@@ -50,6 +50,7 @@ class AlexaLogin:
     outputpath (function): Local path with write access for storing files
     debug (boolean): Enable additional debugging including debug file creation
     otp_secret (string): TOTP Secret key for automatic 2FA filling
+    uuid: (string): Unique 32 char hex to serve as app serial number for registration
 
     """
 
@@ -62,6 +63,7 @@ class AlexaLogin:
         debug: bool = False,
         otp_secret: Text = "",
         oauth: Optional[Dict[Any, Any]] = None,
+        uuid: Optional[Text] = None,
     ) -> None:
         # pylint: disable=too-many-arguments,import-outside-toplevel
         """Set up initial connection and log in."""
@@ -108,6 +110,7 @@ class AlexaLogin:
         self.refresh_token: Optional[Text] = oauth.get("refresh_token")
         self.expires_in: Optional[float] = oauth.get("expires_in")
         self._oauth_lock: asyncio.Lock = asyncio.Lock()
+        self.uuid = uuid  # needed to be unique but repeateable for device registration
 
     @property
     def email(self) -> Text:
@@ -369,7 +372,6 @@ class AlexaLogin:
         Tests:
         - Attempts to get authenticaton and compares to expected login email
         Returns false if unsuccesful getting json or the emails don't match
-        - Checks for existence of csrf cookie
         Returns false if no csrf found; necessary to issue commands
         """
         if self._debug:
@@ -380,7 +382,9 @@ class AlexaLogin:
         if not self._session:
             self._create_session()
         get_resp = await self._session.get(
-            self._prefix + self._url + "/api/bootstrap", cookies=cookies, ssl=self._ssl
+            self._prefix + "amazon.com" + "/api/bootstrap",
+            cookies=cookies,
+            ssl=self._ssl,
         )
         await self._process_resp(get_resp)
         try:
@@ -392,18 +396,34 @@ class AlexaLogin:
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             return False
-        self.customer_id = json.get("authentication", {}).get("customerId")
-        if email != "" and email.lower() == self._email.lower():
-            _LOGGER.debug("Logged in as %s with id: %s", email, self.customer_id)
-            self.stats["login_timestamp"] = datetime.datetime.now()
-            self.stats["api_calls"] = 0
-            return True
-        if email == "":
-            _LOGGER.debug(
-                "Logged in as mobile account %s with %s", email, self.customer_id
+        # Convert from amazon.com domain to native domain
+        if self._url != "amazon.com":
+            await self.get_tokens()
+            await self.exchange_token_for_cookies()
+            get_resp = await self._session.get(
+                self._prefix + self._url + "/api/bootstrap",
             )
+            await self._process_resp(get_resp)
+            try:
+                json = await get_resp.json()
+                email = json["authentication"]["customerEmail"]
+            except (JSONDecodeError, SimpleJSONDecodeError, ContentTypeError) as ex:
+                _LOGGER.debug(
+                    "Not logged in: %s",
+                    EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
+                )
+                return False
+        self.customer_id = json.get("authentication", {}).get("customerId")
+        if (email != "" and email.lower() == self._email.lower()) or email == "":
+            if email != "":
+                _LOGGER.debug("Logged in as %s with id: %s", email, self.customer_id)
+            else:
+                _LOGGER.debug(
+                    "Logged in as mobile account %s with %s", email, self.customer_id
+                )
             self.stats["login_timestamp"] = datetime.datetime.now()
             self.stats["api_calls"] = 0
+            await self.check_domain()
             return True
         _LOGGER.debug("Not logged in due to email mismatch")
         await self.reset()
@@ -480,22 +500,25 @@ class AlexaLogin:
                 self.status["login_successful"] = True
                 _LOGGER.debug("Log in successful with cookies")
                 await self.get_tokens()
+                # await self.refresh_access_token()
+                # await self.exchange_token_for_cookies()
+                await self.get_csrf()
                 await self.save_cookiefile()
                 return
             await self.reset()
         _LOGGER.debug("Using credentials to log in")
         if not self._site:
-            site: URL = URL(f"https://www.{self._url}/ap/signin")
+            site: URL = URL("https://www.amazon.com/ap/signin")
             deviceid: Text = "453535333833343639344134304445313832303944354342344141414342453123413249564c5635564d32573831"
             query = {
-                "openid.return_to": f"https://www.{self._url}/ap/maplanding",
+                "openid.return_to": "https://www.amazon.com/ap/maplanding",
                 "openid.assoc_handle": "amzn_dp_project_dee_ios",
                 "openid.identity": "http://specs.openid.net/auth/2.0/identifier_select",
                 "pageId": "amzn_dp_project_dee_ios",
                 "accountStatusPolicy": "P1",
                 "openid.claimed_id": "http://specs.openid.net/auth/2.0/identifier_select",
                 "openid.mode": "checkid_setup",
-                "openid.ns.oa2": f"http://www.{self._url}/ap/ext/oauth/2",
+                "openid.ns.oa2": "http://www.amazon.com/ap/ext/oauth/2",
                 "openid.oa2.client_id": f"device:{deviceid}",
                 "openid.ns.pape": "http://specs.openid.net/extensions/pape/1.0",
                 "openid.oa2.response_type": "token",
@@ -649,9 +672,9 @@ class AlexaLogin:
                 "device_type": "A2IVLV5VM2W81",
                 "device_name": f"%FIRST_NAME%'s%DUPE_STRATEGY_1ST%{APP_NAME}",
                 "os_version": "11.4.1",
-                "device_serial": "2cf947a5a093d686a30c33cb07a703fd",
-                # may need to be replaced with random 16 bytes; leaving static for now
-                # https://github.com/Apollon77/alexa-cookie/blob/master/lib/proxy.js#L102-L106
+                "device_serial": "2cf947a5a093d686a30c33cb07a703fd"
+                if not self.uuid
+                else self.uuid,
                 "device_model": "iPhone",
                 "app_name": APP_NAME,
                 "software_version": "1",
@@ -669,7 +692,10 @@ class AlexaLogin:
         response = (await response.json()).get("response")
         # _LOGGER.debug("auth response %s with \n%s", response, dumps(data))
         if response.get("success"):
-            _LOGGER.debug("Successfully registered device with Amazon")
+            _LOGGER.debug(
+                "Successfully registered %s device with Amazon",
+                response["success"]["extensions"]["device_info"]["device_name"],
+            )
             if self._debug:
                 _LOGGER.debug("Received registration data:\n%s", dumps(response))
             self.refresh_token = response["success"]["tokens"]["bearer"][
@@ -729,11 +755,15 @@ class AlexaLogin:
         response = await self._session.post(
             "https://api." + self._url + "/auth/token", data=data, headers=headers,
         )
-        # _LOGGER.debug("refresh token response %s with \n%s", response, dumps(data))
         if response.status != 200:
+            if self._debug:
+                _LOGGER.debug("Failed to refresh access token: %s", response)
+            else:
+                _LOGGER.debug("Failed to refresh access token")
             return False
         response = await response.json()
-        # _LOGGER.debug("refresh token json %s ", response)
+        if self._debug:
+            _LOGGER.debug("Refresh token json:\n%s ", response)
         if response.get("access_token"):
             self.access_token = response.get("access_token")
             self.expires_in = datetime.datetime.now().timestamp() + int(
@@ -786,11 +816,17 @@ class AlexaLogin:
             data=data,
             headers=headers,
         )
-        # _LOGGER.debug("exchange token response %s with \n%s", response, dumps(data))
         if response.status != 200:
+            if self._debug:
+                _LOGGER.debug(
+                    "Failed to exchange cookies for refresh token: %s", response
+                )
+            else:
+                _LOGGER.debug("Failed to exchange cookies for refresh token")
             return False
         response = (await response.json()).get("response")
-        # _LOGGER.debug("exchange token json %s ", response)
+        if self._debug:
+            _LOGGER.debug("Exchange cookie json %s ", response)
         for domain, cookies in response["tokens"]["cookies"].items():
             # _LOGGER.debug("updating %s with %s", domain, cookies)
             for item in cookies:
@@ -810,10 +846,82 @@ class AlexaLogin:
                     raw_cookie[cookie_name][name] = value
                 # _LOGGER.debug("updating jar with cookie %s", raw_cookie)
                 self._session.cookie_jar.update_cookies(raw_cookie, URL(domain))
-        _LOGGER.debug(
-            "Cookies successfully exchanged for refresh token for domain %s",
-            response["tokens"]["cookies"].keys(),
+        for domain, cookies in response["tokens"]["cookies"].items():
+            _LOGGER.debug(
+                "%s cookies successfully exchanged for refresh token for domain %s",
+                len(cookies),
+                domain,
+            )
+
+    async def get_csrf(self) -> bool:
+        """Generate csrf if missing.
+
+        Returns
+            bool: True if csrf is found
+
+        """
+        if self._cookies.get("csrf"):
+            _LOGGER.debug("CSRF already exists; no need to discover")
+            return True
+        _LOGGER.debug("Attemping to discover CSRF token")
+        csrf_urls = [
+            "/spa/index.html",
+            "/api/language",
+            "/api/devices-v2/device?cached=false",
+            "/templates/oobe/d-device-pick.handlebars",
+            "/api/strings",
+        ]
+        for url in csrf_urls:
+            response = await self._session.get(f"{self._prefix}{self._url}{url}")
+            if response.status != 200:
+                if self._debug:
+                    _LOGGER.debug("Unable to load page for csrf: %s", response)
+                continue
+            self._prepare_cookies_from_session(self._url)
+            if self._cookies.get("csrf"):
+                _LOGGER.debug("CSRF token found from %s", url)
+                return True
+            _LOGGER.debug("CSRF token not found from %s", url)
+        _LOGGER.debug("No csrf token found")
+        return False
+
+    async def check_domain(self) -> bool:
+        """Check whether logged into appropriate login domain.
+
+        Returns
+            bool: True if in correct domain
+
+        """
+        headers = {
+            "Content-Type": "application/x-www-form-urlencoded",
+            "Accept-Charset": "utf-8",
+            "x-amzn-identity-auth-domain": f"api.{self._url}",
+            "Connection": "keep-alive",
+            "Accept": "*/*",
+            "User-Agent": USER_AGENT,
+            "Accept-Language": "en-US",
+        }
+        response = await self._session.get(
+            f"{self._prefix}{self._url}/api/users/me?platform=ios&version=2.2.223830.0",
+            headers=headers,
         )
+        if response.status != 200:
+            if self._debug:
+                _LOGGER.debug("Unable to check for domain; proceeding:\n%s", response)
+            return True
+        response = await response.json()
+        domain = response.get("marketPlaceDomainName")
+        if self._url not in domain:
+            _LOGGER.warning(
+                "Domain %s does not match reported account domain %s; functionality is not likely to work, please fix",
+                self.url,
+                domain,
+            )
+            return False
+        _LOGGER.debug(
+            "Domain %s matches reported account domain: %s", self._url, domain
+        )
+        return True
 
     async def _process_resp(self, resp) -> Text:
         if resp.history:
@@ -1056,14 +1164,15 @@ class AlexaLogin:
             status["approval_status"] = approval_status.get("value")
         else:
             _LOGGER.debug("Captcha/2FA not requested; confirming login.")
+            query = site_url.query
+            self.access_token = query.get("openid.oa2.access_token")
             if await self.test_loggedin():
-                query = site_url.query
-                self.access_token = query.get("openid.oa2.access_token")
                 _LOGGER.debug(
                     "Login confirmed; saving cookie to %s", self._cookiefile[0]
                 )
                 status["login_successful"] = True
                 await self.get_tokens()
+                await self.get_csrf()
                 await self.save_cookiefile()
                 #  remove extraneous Content-Type to avoid 500 errors
                 self._headers.pop("Content-Type", None)
